# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from cms.api import get_page_draft
from cms.cms_toolbars import PAGE_MENU_SECOND_BREAK
from cms.toolbar.items import Break
from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool
from cms.utils import get_cms_setting
from cms.utils.i18n import get_language_list, get_language_object
from django.core.urlresolvers import NoReverseMatch, reverse
from django.utils.translation import ugettext_lazy as _

from .models import PageTags, TitleTags

try:
    from cms.utils.permissions import has_page_permission
    has_page_change_permission = None
except ImportError:
    from cms.utils.permissions import has_page_change_permission


PAGE_TAGS_MENU_TITLE = _('Tags')
PAGE_TAGS_ITEM_TITLE = _('Common')


@toolbar_pool.register
class PageTagsToolbar(CMSToolbar):
    def populate(self):
        # always use draft if we have a page
        self.page = get_page_draft(self.request.current_page)

        if not self.page:
            # Nothing to do
            return

        # check global permissions if CMS_PERMISSIONS is active
        if get_cms_setting('PERMISSION'):
            if not has_page_change_permission:
                has_global_current_page_change_permission = has_page_permission(
                    self.request.user, self.request.current_page, 'change'
                )
            else:
                has_global_current_page_change_permission = has_page_change_permission(
                    self.request
                )
        else:
            has_global_current_page_change_permission = False
            # check if user has page edit permission
        if not has_page_change_permission:
            can_change = (self.request.current_page and
                          self.request.current_page.has_change_permission(self.request.user))
        else:
            can_change = (self.request.current_page and
                          self.request.current_page.has_change_permission(self.request))
        if has_global_current_page_change_permission or can_change:
            not_edit_mode = not self.toolbar.edit_mode
            tags_menu = self.toolbar.get_or_create_menu('page')
            super_item = tags_menu.find_first(Break, identifier=PAGE_MENU_SECOND_BREAK) + 1
            tags_menu = tags_menu.get_or_create_menu(
                'pagetags', PAGE_TAGS_MENU_TITLE, position=super_item
            )
            position = 0
            # Page tags
            try:
                page_extension = PageTags.objects.get(extended_object_id=self.page.pk)
            except PageTags.DoesNotExist:
                page_extension = None
            try:
                if page_extension:
                    url = reverse('admin:djangocms_page_tags_pagetags_change',
                                  args=(page_extension.pk,))
                else:
                    url = '%s?extended_object=%s' % (
                        reverse('admin:djangocms_page_tags_pagetags_add'),
                        self.page.pk
                    )
            except NoReverseMatch:  # pragma: no cover
                # not in urls
                pass
            else:
                tags_menu.add_modal_item(PAGE_TAGS_ITEM_TITLE, url=url, disabled=not_edit_mode,
                                         position=position)
            # Title tags
            for title in self.page.title_set.filter(
                language__in=get_language_list(self.page.site_id)
            ):
                try:
                    title_extension = TitleTags.objects.get(extended_object_id=title.pk)
                except TitleTags.DoesNotExist:
                    title_extension = None
                try:
                    if title_extension:
                        url = reverse('admin:djangocms_page_tags_titletags_change',
                                      args=(title_extension.pk,))
                    else:
                        url = '%s?extended_object=%s' % (
                            reverse('admin:djangocms_page_tags_titletags_add'),
                            title.pk)
                except NoReverseMatch:  # pragma: no cover
                    # not in urls
                    pass
                else:
                    position += 1
                    language = get_language_object(title.language)
                    tags_menu.add_modal_item(language['name'], url=url, disabled=not_edit_mode,
                                             position=position)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from classytags.arguments import Argument
from classytags.core import Options
from classytags.helpers import AsTag, InclusionTag
from django import template

from ..utils import get_page_tags_from_request

register = template.Library()


class IncludePageTagsList(InclusionTag):
    template = 'djangocms_page_tags/templatetags/page_tags.html'
    name = 'include_page_tags'
    title = False

    options = Options(
        Argument('page_lookup'),
        Argument('lang', required=False, default=None),
        Argument('site', required=False, default=None),
    )

    def get_context(self, context, page_lookup, lang, site):
        request = context.get('request', False)
        if not request:  # pragma: no cover
            return {'tags_list': ''}
        tags_list = get_page_tags_from_request(request, page_lookup, lang, site,
                                               self.title)
        if tags_list:
            return {'tags_list': tags_list}
        return {'tags_list': ''}
register.tag(IncludePageTagsList)


class IncludeTitleTagsList(IncludePageTagsList):
    template = 'djangocms_page_tags/templatetags/title_tags.html'
    name = 'include_title_tags'
    title = True
register.tag(IncludeTitleTagsList)


class PageTagsList(AsTag):
    name = 'page_tags'
    title = False

    options = Options(
        Argument('page_lookup'),
        Argument('lang', required=False, default=None),
        Argument('site', required=False, default=None),
        'as',
        Argument('varname', required=True, resolve=False)
    )

    def get_value(self, context, page_lookup, lang, site):
        request = context.get('request', False)
        if not request:  # pragma: no cover
            return ''
        return get_page_tags_from_request(request, page_lookup, lang, site,
                                          self.title)
register.tag(PageTagsList)


class TitleTagsList(PageTagsList):
    name = 'title_tags'
    title = True
register.tag(TitleTagsList)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from cms.extensions import PageExtensionAdmin, TitleExtensionAdmin
from django.conf import settings
from django.contrib import admin

from .models import PageTags, TitleTags


class PageTagsAdmin(PageExtensionAdmin):

    class Media:
        css = {
            'all': ('%sdjangocms_page_tags/css/%s' % (
                settings.STATIC_URL, 'djangocms_page_tags_admin.css'),)
        }
admin.site.register(PageTags, PageTagsAdmin)


class TitleTagsAdmin(TitleExtensionAdmin):

    class Media:
        css = {
            'all': ('%sdjangocms_page_tags/css/%s' % (
                settings.STATIC_URL, 'djangocms_page_tags_admin.css'),)
        }
admin.site.register(TitleTags, TitleTagsAdmin)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from cms.extensions import PageExtension, TitleExtension
from cms.extensions.extension_pool import extension_pool
from cms.models import Page, Title
from django.core.cache import cache
from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _
from taggit_autosuggest.managers import TaggableManager

from .utils import get_cache_key


class PageTags(PageExtension):
    tags = TaggableManager()

    def copy_relations(self, oldinstance, language):
        """ Needed to copy tags when publishing page """
        self.tags.set(*oldinstance.tags.all())

    class Meta:
        verbose_name = _('Page tags (all languages)')
extension_pool.register(PageTags)


class TitleTags(TitleExtension):
    tags = TaggableManager()

    def copy_relations(self, oldinstance, language):
        """ Needed to copy tags when publishing page """
        self.tags.set(*oldinstance.tags.all())

    class Meta:
        verbose_name = _('Page tags (language-dependent)')
extension_pool.register(TitleTags)


# Cache cleanup when deleting pages / editing page extensions
@receiver(pre_delete, sender=Page)
def cleanup_page(sender, instance, **kwargs):
    key = get_cache_key(
        None, instance, '', instance.site_id, False
    )
    cache.delete(key)


@receiver(pre_delete, sender=Title)
def cleanup_title(sender, instance, **kwargs):
    key = get_cache_key(
        None, instance.page, instance.language, instance.page.site_id, True
    )
    cache.delete(key)


@receiver(post_save, sender=PageTags)
def cleanup_pagetags(sender, instance, **kwargs):
    key = get_cache_key(
        None, instance.extended_object, '', instance.extended_object.site_id, False
    )
    cache.delete(key)


@receiver(post_save, sender=TitleTags)
def cleanup_titletags(sender, instance, **kwargs):
    key = get_cache_key(
        None, instance.extended_object.page, instance.extended_object.language,
        instance.extended_object.page.site_id, True
    )
    cache.delete(key)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import taggit_autosuggest.managers


class Migration(migrations.Migration):

    dependencies = [
        ('taggit', '__latest__'),
        ('cms', '__latest__'),
    ]

    operations = [
        migrations.CreateModel(
            name='PageTags',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, serialize=False, primary_key=True)),
                ('extended_object', models.OneToOneField(to='cms.Page', editable=False)),
                ('public_extension', models.OneToOneField(to='djangocms_page_tags.PageTags', related_name='draft_extension', null=True, editable=False)),
                ('tags', taggit_autosuggest.managers.TaggableManager(through='taggit.TaggedItem', to='taggit.Tag', verbose_name='Tags', help_text='A comma-separated list of tags.')),
            ],
            options={
                'verbose_name': 'Page tags (all languages)',
            },
        ),
        migrations.CreateModel(
            name='TitleTags',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, serialize=False, primary_key=True)),
                ('extended_object', models.OneToOneField(to='cms.Title', editable=False)),
                ('public_extension', models.OneToOneField(to='djangocms_page_tags.TitleTags', related_name='draft_extension', null=True, editable=False)),
                ('tags', taggit_autosuggest.managers.TaggableManager(through='taggit.TaggedItem', to='taggit.Tag', verbose_name='Tags', help_text='A comma-separated list of tags.')),
            ],
            options={
                'verbose_name': 'Page tags (language-dependent)',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals


def get_cache_key(request, page, lang, site_id, title):
    """
    Create the cache key for the current page and tag type
    """
    from cms.cache import _get_cache_key
    from cms.templatetags.cms_tags import _get_page_by_untyped_arg

    from cms.models import Page
    if not isinstance(page, Page):
        page = _get_page_by_untyped_arg(page, request, site_id)
    if not site_id:
        site_id = page.site_id
    if not title:
        return _get_cache_key('page_tags', page, '', site_id) + '_type:tags_list'
    else:
        return _get_cache_key('title_tags', page, lang, site_id) + '_type:tags_list'


def get_page_tags(page):
    """
    Retrieves all the tags for a Page instance.

    :param page: a Page instance

    :return: list or queryset of attached tags
    :type: List
    """
    from .models import PageTags
    try:
        return page.pagetags.tags.all()
    except PageTags.DoesNotExist:
        return []


def page_has_tag(page, tag):
    """
    Check if a Page object is associated with the given tag.

    :param page: a Page instance
    :param tag: a Tag instance or a slug string.

    :return: whether the Page instance has the given tag attached (False if no Page or no
             attached PageTags exists)
    :type: Boolean
    """
    from .models import PageTags
    if hasattr(tag, 'slug'):
        slug = tag.slug
    else:
        slug = tag
    try:
        return page.pagetags.tags.filter(slug=slug).exists()
    except PageTags.DoesNotExist:
        return False


def get_title_tags(page, lang):
    """
    Retrieves all the tags for a Title (given as page and language).
    This function does not use fallbacks to retrieve title object.

    :param page: a Page instance
    :param lang: a language code

    :return: list or queryset of attached tags
    :type: List
    """
    from .models import TitleTags
    try:
        return page.get_title_obj(language=lang, fallback=False).titletags.tags.all()
    except TitleTags.DoesNotExist:
        return []


def title_has_tag(page, lang, tag):
    """
    Check if a Title object is associated with the given tag.
    This function does not use fallbacks to retrieve title object.

    :param page: a Page instance
    :param lang: a language code
    :param tag: a Tag instance or a slug string.

    :return: whether the Title instance has the given tag attached (False if no Title or no
             attached TitleTags exists)
    :type: Boolean
    """
    from .models import TitleTags
    if hasattr(tag, 'slug'):
        slug = tag.slug
    else:
        slug = tag
    try:
        return page.get_title_obj(
            language=lang, fallback=False
        ).titletags.tags.filter(slug=slug).exists()
    except TitleTags.DoesNotExist:
        return False


def get_page_tags_from_request(request, page_lookup, lang, site, title=False):
    """
    Get the list of tags attached to a Page or a Title from a request  from usual
    `page_lookup` parameters.

    :param request: request object
    :param page_lookup: a valid page_lookup argument
    :param lang: a language code
    :param site: a site id
    :param title: a boolean to extract the Page (if False) or Title instance

    :return: list of tags
    :type: List
    """
    from cms.templatetags.cms_tags import _get_page_by_untyped_arg
    from cms.utils import get_language_from_request, get_cms_setting, get_site_id
    from django.core.cache import cache

    site_id = get_site_id(site)
    if lang is None:
        lang = get_language_from_request(request)
    cache_key = get_cache_key(request, page_lookup, lang, site, title)
    tags_list = cache.get(cache_key)
    if not tags_list:
        page = _get_page_by_untyped_arg(page_lookup, request, site_id)
        if page:
            if title:
                tags_list = get_title_tags(page, lang)
            else:
                tags_list = get_page_tags(page)
            cache.set(cache_key, tags_list, timeout=get_cms_setting('CACHE_DURATIONS')['content'])
    if not tags_list:
        tags_list = ()
    return tags_list


def get_title_tags_from_request(request, page_lookup, lang, site):
    """
    Get the list of tags attached to a Title from a request from usual
    `page_lookup` parameters.

    :param request: request object
    :param page_lookup: a valid page_lookup argument
    :param lang: a language code
    :param site: a site id

    :return: list of tags attached to the given Title
    :type: List
    """
    return get_page_tags_from_request(request, page_lookup, lang, site, True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import djangocms_page_tags

from setuptools import setup

version = djangocms_page_tags.__version__

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    print('You probably want to also tag the version now:')
    print('  git tag -a %s -m "version %s"' % (version, version))
    print('  git push --tags')
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='djangocms-page-tags',
    version=version,
    description='Tagged pages for django CMS 3',
    long_description=readme + '\n\n' + history,
    author='Iacopo Spalletti',
    author_email='i.spalletti@nephila.it',
    url='https://github.com/nephila/djangocms-page-tags',
    packages=[
        'djangocms_page_tags',
    ],
    include_package_data=True,
    install_requires=(
        'django-cms>=3.2',
        'django-taggit>=0.11.2',
        'django-taggit-autosuggest',
        'django-classy-tags>=0.3.4.1',
    ),
    license='BSD',
    zip_safe=False,
    test_suite='cms_helper.run',
    keywords='djangocms-page-tags',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
